from typing import Dict, Union, List
from jsonschema import Draft7Validator
from flask import request
from rest_utilities.response.builder import build_response

import functools


def validate_schema(json: Union[List, Dict], schema) -> Union[List, bool]:
    """ validates a json  object according to a jsonschema
        INPUT -  json: json to validate
                schema: jsonschema defininig valid json structure.
                See: https://json-schema.org/
    """
    validator = Draft7Validator(schema)

    errors = list(map(lambda e: e.message, validator.iter_errors(json)))

    return errors if len(errors) > 0 else False


def valid_request(schema=None):
    """Decorates a flask route to validate the request json object according
    to a jsonschema. If the json is not valid returns a response with
    the errors.
    INPUT - jsonschema defining valid json structure.
            See: https://json-schema.org
    """

    def decorator(func):

        @functools.wraps(func)
        def wrapper_valid_json(*args, **kwargs):
            # validate json
            request_data = request.get_json(silent=True)

            schema_to_validate = func.schema if hasattr(func, 'schema') else schema

            if schema_to_validate is None:
                return build_response('Schema to validate is missing', 500)

            errors = validate_schema(request_data, schema_to_validate)

            # if json is valid, call route function
            if not errors:
                return func(*args, **kwargs)
            # else return errors on route
            else:
                return build_response(errors, 400, 'Some fields are wrong')

        return wrapper_valid_json

    return decorator
